﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing; //【Point】

namespace NTL.KinectLib
{
    //*************************************************************************
    //*************************************************************************
    //【CLASS】『FIFO』【FIFOを実現する】
    //*************************************************************************
    //*************************************************************************
    public class FIFO
    {
        private int nFIFO;
        private double F1;
        private int ptr = 0;
        private double Offset;
        private int nOffset;   //【オフセットを判断するサンプル数】
        public double[] dFIFO;
        public Point[] Points;
        private double max;
        private double min;

        //***************************************************************
        //【Initialize】
        //***************************************************************
        //【パラメータ】n:FIFOの総サンプル数  nO:DCオフセットを判定するサンプル個数
        public void Initialize(int n, int nO)
        {
            max = -32767;
            min = 32767;
            ptr = 0;
            nFIFO = n;
            F1 = 1.0 / (double)nO;
            dFIFO = new double[n];
            nOffset = nO;

            Points = new Point[n];
            for (var i = 0; i < n; i++) { dFIFO[i] = 0.0; }
        }

        //***************************************************************
        //【LoadDataToFIFO】
        //***************************************************************
        public double LoadDataToFIFO(double DataIn)
        {
            int i, j;
            double d, sum;
            double DataOut = dFIFO[ptr];
            dFIFO[ptr] = DataIn;
            max = -1.0D + 105;
            min = 1.0D + 105;
            for (i = 0; i < nFIFO; i++)
            {
                d = dFIFO[i];
                if (d > max) { max = d; }
                if (d < min) { min = d; }
            }
            sum = 0.0;
            for (i = 0, j = ptr; i < nOffset; i++)
            {
                sum += dFIFO[j];
                j--; if (j < 0) { j = nFIFO - 1; }
            }
            Offset = sum * F1;
            ptr++;
            if (ptr == nFIFO) { ptr = 0; }
            return DataOut;
        }

        //***************************************************************
        //【OffsetLevel】
        //***************************************************************
        public double OffsetLevel
        {
            get { return Offset; }
        }

        //***************************************************************
        //【Pointer】
        //***************************************************************
        public int Pointer
        {
            get { return ptr; }
        }

        //***************************************************************
        //【Max】
        //***************************************************************
        public double Max
        {
            get { return max; }
        }

        //***************************************************************
        //【Min】
        //***************************************************************
        public double Min
        {
            get { return min; }
        }
    }
}
