﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace NTL.KinectLib
{
    public class MovingAverageFilter
    {
        private const int cWidth = 640;
        private const int cHeight = 480;
        private const int cSpan = 40;
        private int nWidth = cWidth;
        private int nHeight = cHeight;
        private int nSpan = cSpan;
        private double nSpan2 = 1.0 / (double)(cSpan * cSpan);
        private double[] FIFOx = new double[cSpan];           //【1次元水平方向FIFO、サイズはNTL_nSpan】
        private double[] FIFOxy = new double[cWidth * cSpan]; //【2次元矩形FIFO、サイズはNTL_nSpan*KS_ColorFrameWidth】
        private double[] SumXY = new double[cWidth];          //【矩形領域移動平均、サイズはKS_ColorFrameWidth】
        public double[] BufferA = new double[cWidth * cHeight];
        public double[] BufferB = new double[cWidth * cHeight];
        private int pX, pXY, pY, pB, nF, nB;
        private double cSumX;  //【水平方向累積加算値】
        private double cSumXY; //【矩形小領域内累積加算値】
        private double vmax, vmin;

        //*******************************************************
        //【MovingAverage】
        //*******************************************************
        public void Initialize(int width, int height, int span)
        {
            nWidth=width;
            nHeight=height;
            nSpan=span;
            nSpan2 = 1.0 / (double)(nSpan * nSpan);
            FIFOx = new double[nSpan];           //【1次元水平方向FIFO、サイズはNTL_nSpan】
            FIFOxy = new double[nWidth * nSpan]; //【2次元矩形FIFO、サイズはNTL_nSpan*KS_ColorFrameWidth】
            SumXY = new double[nWidth];          //【矩形領域移動平均、サイズはKS_ColorFrameWidth】
            BufferA = new double[nWidth * nHeight];
            BufferB = new double[nWidth * nHeight];
            init();
        }
        
        public void Initialize()
        {
            init();
        }

        private void init()
        {
            int i;
            for (i = 0; i < nSpan; i++) { FIFOx[i] = 0; } //【FIFOxの0クリア】
            for (i = 0; i < nWidth * nSpan; i++) { FIFOxy[i] = 0; }
            for (i = 0; i < nWidth; i++) { SumXY[i] = 0; }
            cSumX = cSumXY = 0;
            pX = pXY = pY = pB = 0;
            nF = nSpan * nWidth;
            nB = nWidth * nHeight;
            vmax = -1E10;
            vmin = 1E10;
        }

        public double MovingAverage(double Data)
        {
            double v;
            //①【FIFOxの移動平均処理】【水平方向】【スキャンライン】
            cSumX += Data - FIFOx[pX];      //【差分計算から累積加算値を求める】
            FIFOx[pX] = Data;               //【新規データをFIFOに保存する】
            pX++; if (pX == nSpan) pX = 0; //【FIFOx用ポインタ(0～n-1)の更新処理】

            //②【FIFOxyの移動平均処理】【垂直方向】【スクリーン横幅分並列処理】
            SumXY[pY] += cSumX - FIFOxy[pXY]; //【差分計算から累積加算値を求める】
            FIFOxy[pXY] = cSumX;              //【新規データをFIFOに保存する】
            cSumXY = SumXY[pY];               //【n×n画素の矩形領域の累積加算値⇒add】
            pXY++; if (pXY == nF) pXY = 0;    //【FIFOxy用ポインタ(0～n*width-1)の更新処理】
            pY++; if (pY == nWidth) pY = 0;   //【SumXY用ポインタ(0～width-1)の更新処理】

            //③【水平方向シフト量をフレームバッファに書き込む】
            v = cSumXY * nSpan2;
            BufferA[pB] = v;
            pB++; if (pB == nB) { pB = 0; }
            if (v < vmin) { vmin = v; }
            if (v > vmax) { vmax = v; }
            return v;
        }

        public void CopyBuffer()
        {
            for (int i = 0; i < nB; i++)
            {
                BufferB[i] = BufferA[i];
            }
        }

        public double Max
        {
            get { return vmax; }
        }

        public double Min
        {
            get { return vmin; }
        }

        public int Width
        {
            get { return nWidth; }
        }

        public int Height
        {
            get { return nHeight; }
        }

        public int Span
        {
            get { return nSpan; }
        }

    }
}
